export type UtilsArraySignType = '=' | '>' | '>=' | '<' | '<=' | '!=' | 'indexOf';
export type OrderType = 'desc' | 'asc';
export type ColorType = 'rgb' | 'binary'
export interface UtilsArrayWhereOptions {
  //要过滤的值
  value: string | number,
  //如果数据内是对象需要过滤的字段名称
  column?: string,
  //过滤方式，默认 "="
  sign?: UtilsArraySignType
};

export interface IntensityOptions {
  //色阶 ex:{0.1:red,0.5:'#fff'}
  gradient?: object;
  //最大Size
  maxSize?: number;
  //最小Size
  minSize?: number;
  //数据最大值
  max: number;
  //需要获取颜色的类型
  colorType?: ColorType;
}

export interface UtilsArray {
  /**
   * 数组过滤
   * 
   * ex: utils.Array.where([{name:'zs'}],'zs','name','=')
   * ex: utils.Array.where([{name:'zs'}],'zs','name')
   *
   * @template T
   * @param {Array<T>} array 需要过滤的数据
   * @param {(string | number)} value 要过滤的值
   * @param {string} [column] 如果数据内是对象需要过滤的字段名称
   * @param {UtilsArraySignType} [sign] 过滤方式，默认 "="
   * @return {Array<T>}  {Array<T>} 过滤后的数据
   * @memberof UtilsArray
   */
  where<T = any>(
    array: Array<T>,
    value: string | number,
    column?: string,
    sign?: UtilsArraySignType): Array<T>;

  /**
   * 数组过滤
   * 
   * utils.Array.where([{name:'zs'}],{value:'zs','column':'name',sign:'='})
   *
   * @template T
   * @param {Array<T>} array 需要过滤的数据
   * @param {UtilsArrayWhereOptions} options 选项
   * @return {Array<T>}  {Array<T>}
   * @memberof UtilsArray
   */
  where<T = any>(
    array: Array<T>,
    options: UtilsArrayWhereOptions): Array<T>;


  /**
   * 数组过滤返回第一条，若无匹配返回空对象
   *
   * ex: utils.Array.whereFirst([{name:'zs'}],'zs','name','=')
   * ex: utils.Array.whereFirst([{name:'zs'}],'zs','name')
   *
   * @template T
   * @param {Array<T>} array 需要过滤的数据
   * @param {(string | number)} value 要过滤的值
   * @param {string} [column] 如果数据内是对象需要过滤的字段名称
   * @param {UtilsArraySignType} [sign] 过滤方式，默认 "="
   * @return {T}  {T} 过滤后的数据
   * @memberof UtilsArray
   */
  whereFirst<T = any>(
    array: Array<T>,
    value: string | number,
    column?: string,
    sign?: UtilsArraySignType): T;


  /**
   * 数组过滤返回第一条，若无匹配返回空对象
   *
   * utils.Array.whereFirst([{name:'zs'}],{value:'zs','column':'name',sign:'='})
   *
   * @template T
   * @param {Array<T>} array 需要过滤的数据
   * @param {UtilsArrayWhereOptions} options 选项
   * @return {Array<T>}  {Array<T>}
   * @memberof UtilsArray
   */
  whereFirst<T = any>(
    array: Array<T>,
    options: UtilsArrayWhereOptions): T;


  /**
   * 判断元素是否在数组中存在
   *
   * @template T
   * @param {Array<T>} array 需要过滤的数据
   * @param {(string | number)} value 要过滤的值
   * @param {string} [column] 如果数据内是对象需要过滤的字段名称
   * @param {UtilsArraySignType} [sign] 过滤方式，默认 "="
   * @return {boolean}  {boolean}
   * @memberof UtilsArray
   */
  contains<T = any>(
    array: Array<T>,
    value: string | number,
    column?: string,
    sign?: UtilsArraySignType): boolean;


  /**
   * 判断元素是否在数组中存在
   *
   * @template T
   * @param {Array<T>} array 需要判断的数据
   * @param {UtilsArrayWhereOptions} options 选项
   * @return {boolean}  {boolean}
   * @memberof UtilsArray
   */
  contains<T = any>(
    array: Array<T>,
    options: UtilsArrayWhereOptions): boolean;



  /**
   * 数组排序,返回新的数组
   *
   * @template T
   * @param {Array<T>} array 需要排序的数组
   * @param {OrderType} type 排序方式
   * @param {string} [column] 排序字段（如元素为对象需要指明字段名称）
   * @return {Array<T>}  {Array<T>}
   * @memberof UtilsArray
   */
  order<T = any>(
    array: Array<T>,
    type: OrderType,
    column?: string): Array<T>;


  /**
   * 数组去重
   *
   * @template T
   * @param {Array<T>} array 需要去重的数组
   * @param {string} [column] 按指定字段去重（如元素为对象需要指明字段名称）
   * @return {Array<T>}  {Array<T>}
   * @memberof UtilsArray
   */
  unique<T = any>(array: Array<T>, column?: string): Array<T>;

  /**
   * 获取匹配的第一个元素索引值，未匹配上返回 -1
   *
   * @template T
   * @param {Array<T>} array 要匹配的数据源
   * @param {(string | number | boolean | null)} value 要匹配的值
   * @param {string} [column] 如果是数组对象指定列名
   * @return {number}  {number}
   * @memberof UtilsArray
   */
  indexOf<T = any>(array: Array<T>, value: string | number | boolean | null, column?: string): number;
};

interface UtilsString {
  /**
   * 要替换的模板
   * ex:const value = format(`abc{0}ccc{0},{1}test`,122,'3')
   * value:abc122ccc122,3test
   * @param {string} template
   * @param {string|number} args
   * @return {string}  返回替换后的字符串
   * @memberof UtilsString 
   */
  format(template: string, ...args): string;


  /**
   * 删除HTML标签
   *
   * @param {string} html 需要删除HTML标签的HTML片段
   * @return {string}  {string}
   * @memberof UtilsString
   */
  removeHTML(html: string): string;

  /**
   * 截取字符串并拼接字符
   *
   * @param {string} content 需要截取的字符串
   * @param {number} len 截取字符串长度
   * @param {string} suffix 截取字符串后需要拼接的字符
   * @return {string}  返回处理好的字符
   * @memberof UtilsString
   */
  subString(content: string, len: number, suffix: string): string;


  /**
   * 字符串转换parseFloat并保留指定小数位
   *
   * @param {string} content 要操作的字符串
   * @param {number} len 保留小数位长度
   * @param {boolean} [fill=true] 如果小数位长度不足是否需要补齐 default =true
   * @return {string}  转换后的值
   * @memberof UtilsString
   */
  toFixed(content: string, len: number, fill: boolean = true): string;


  /**
   * 去掉前后空格
   *
   * @param {string} content 待处理的字符
   * @return {string}  去除空格字符
   * @memberof UtilsString
   */
  trim(content: string): string;


  /**
   * 替换字符
   *
   * @param {string} content 要操作的字符串
   * @param {string} replacestr  被替换字符串
   * @param {string} newstr 用来替换的字符串
   * @return {string}  替换后的字符串
   * @memberof UtilsString
   */
  replaceAll(content: string, replacestr: string, newstr: string): string;


  /**
   * 日期字符串转换日期
   *
   * @param {string} content 需要转换为日期的字符串
   * @param {string} [split='-'] 字符串分隔符
   * @return {Date}  返回日期
   * @memberof UtilsString
   */
  toDate(content: string, split?: string = '-'): Date | null;
};

interface UtilsDate {
  /**
   * 日期格式化
   *
   * @param {Date} date 需要格式化的日期
   * @param {string} fmt 转换的格式 如 yyyy-MM-dd
   * @return {string}  {string}
   * @memberof UtilsDate
   */
  format(date: Date, fmt: string): string;
};
export interface UtilsObject {
  /**
   * 深度拷贝对象
   *
   * @param {boolean} deep
   * @param {...object} args
   * @return {*}  {object}
   * @memberof UtilsObject
   */
  extend<T = object>(deep: boolean, ...args: object): T;
  /**
   * 深度拷贝对象
   *
   * @param {...object} args
   * @return {*}  {object}
   * @memberof UtilsObject
   */
  extend<T = object>(...args: object): T;
}

export interface Intensity {

  /**
   * 获取色阶值
   *
   * @param {number} value 需要获取计算后色阶的值
   * @return {string}  返回计算后的颜色
   * @memberof Intensity
   */
  getColor(value: number): string;

  /**
   * 获取计算后得大小(适用场景:上图后计算气泡半径)
   *
   * @param {number} value 需要获取计算后size的值
   * @return {string}  返回计算后的大小
   * @memberof Intensity
   */
  getSize(value: number): number;
}

export interface UtilsColor {
  /**
   * 将rgb颜色转换成二进制网页色
   *
   * @param {string} rgbColor rgb颜色
   * @return {string}  二进制网页色
   * @memberof UtilsColor
   */
  rgb2binary(rgbColor: string): string;

  /**
   * 获取渐变色阶或size
   *
   * @param {IntensityOptions} options
   * @return {Intensity}  {Intensity}
   * @memberof UtilsColor
   */
  intensity(options: IntensityOptions): Intensity;
};


export interface UtilsStatic {
  get Object(): UtilsObject;
  get Array(): UtilsArray;
  get Date(): UtilsDate;
  get String(): UtilsString;
  get Utils(): UtilsColor;
  get Color(): UtilsColor;
}
declare const Utils: UtilsStatic;

export default Utils;
